define([
    "dojo/_base/declare",
    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/form/_FormValueMixin",
    "dijit/TooltipDialog",
    "dijit/form/DropDownButton",
    "dijit/form/Button",
    "dojox/form/CheckedMultiSelect",
    "dojo/when",
    "epi/dependency",
    "dojo/text!./MultiSelectList.html"
], function (
    declare,
    _WidgetBase,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _FormValueMixin,
    TooltipDialog,
    DropDownButton,
    Button,
    CheckedMultiSelect,
    when,
    dependency,
    template
) {

        return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _FormValueMixin], {
            templateString: template,
            options: null,
            label: "Filter",
            multiple: true,
            baseClass: "digi-multi-select-list",
            translationStore: null,

            postCreate: function () {
                var registry = dependency.resolve("epi.storeregistry");
                this.translationStore = this.translationStore || registry.get("digizuite.translation");

                when(this.translationStore.executeMethod('GetTranslationsOnDemand', '?key=' + '/digizuite/generalcaptions/applyfilter'),
                    function (response) {
                        if (this.applyFilter !== undefined) {
                            this.applyFilter.set("label", response);
                        }
                    }.bind(this));
            },

            _onApplyFilter: function () {
                var value = this.select.get("value");

                if (!Array.isArray(value)) {
                    this.dropDown.set("label", this.select.getOptions(value).label);
                } else if (value.length === 1) {
                    this.dropDown.set("label", this.select.getOptions(value[0].toString()).label);
                } else if (value.length > 1) {
                    this.dropDown.set("label", this.label + " +" + value.length);
                } else {
                    this.dropDown.set("label", this.label);
                }

                this.set("value", value);
            },

            _setOptionsAttr: function (options) {
                this._set("options", options);

                options.forEach(function (option) {
                    this.select.addOption(option);
                }.bind(this));
            },

            reset: function (defaultLabel) {
                this._resetValue = [];
                this.inherited(arguments);
                this.set('label', defaultLabel);
                this.dropDown.set('label', defaultLabel);
                this.select.value = [];
                this.select.checked = false;

                if (!this._resetValue || !this._resetValue.length) {
                    this.select._updateSelection();
                }
            },

            setDefaultSelection: function () {
                // select default
                for (const item of this.options) {
                    if (item.isDefault !== undefined)
                        item.selected = item.isDefault;
                    else
                        item.selected = false;
                }
            }
        });

    });