define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-construct",
    "dojo/dom-class",
    "dojo/on",
    "dojo/mouse",
    "dojo/when",
    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/Tooltip",
    "dijit/TooltipDialog",
    "dijit/popup",
    "dijit/_FocusMixin",
    "dijit/registry",
    "dgrid/extensions/DijitRegistry",
    // dgrid
    "dgrid/Keyboard",
    "dgrid/OnDemandList",
    "dgrid/Selection",
    "epi/shell/dgrid/Formatter",
    "epi/shell/dgrid/selection/Extensions",
    "epi/shell/dnd/Source",
    "epi-cms/dgrid/DnD",
    "epi-cms/dgrid/WithContextMenu",
    "epi-cms/dgrid/formatters",

    "epi/shell/dgrid/WidgetRow",
    "epi-cms/widget/ContentList",
    "epi/dependency",
    "epi-cms/widget/_HasChildDialogMixin",
    "dojo/text!./ContentItemTemplate.html",
    "dojo/text!./ContentItemTooltipTemplate.html"
], 
function(
    declare,
    lang,
    domConstruct,
    domClass,
    on,
    mouse,
    when,
    _WidgetBase,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    Tooltip,
    TooltipDialog,
    popup,
    _FocusMixin,
    registry,
    DijitRegistry,
    Keyboard,
    OnDemandList,
    Selection,
    Formatter,
    SelectionExtensions,
    Source,
    DnD,
    WithContextMenu,
    formatters,
    WidgetRow,
    EpiContentList,
    dependency,
    _HasChildDialogMixin,
    ContentItemTemplate,
    ContentItemTooltipTemplate
){
    var ContentItem = declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _FocusMixin, _HasChildDialogMixin], {
        templateString: ContentItemTemplate,
        baseClass: "digi-content-list__content-item",
    
        postMixInProperties: function () {
            this.inherited(arguments);

            this.itemName = this.item.name;
            this.itemTitle = this.item.name;

            // Hack, use the Dnd mixin to get the correct dnd types
            // Perhaps we should copy the _getDndType method instead?
            this._dnd = new DnD();
        },
        buildRendering: function () {
            this.inherited(arguments);

            // Setup Dnd
            // We need to different sources as the crops are located under another 
            // node and dojo dnd expects all dnd'able nodes to be under the same parent
            this._source = new Source(this.rowNode, {
                singular: true,
                copyOnly: true,
                selfCopy: false,
                creator: this._createDndItem.bind(this)
            });

            this._cropsDndSource = new Source(this.cropsNode, {
                singular: true,
                copyOnly: true,
                selfCopy: false,
                creator: this._createDndItem.bind(this)
            });

            // Add the current item to the source
            this._source.setItem(this.dndItemNode.id, {
                name: this.item.name,
                data: this.item,
                type: this._dnd._getDndType(this.item)
            });
            
            // TODO: Use the formatters to render contentIcon and content name
            if(this.item.thumbnailUrl) {
                var thumbnailNode = domConstruct.toDom(formatters.thumbnail(this.item.thumbnailUrl));
                domConstruct.place(thumbnailNode, this.thumbnailNode, "first");
            }

            // config object
            const config = {
                attributes: true,
                attributeOldValue: true,
                attributeFilter: ['class'],
                characterData: false,
                characterDataOldValue: false,
                childList: false,
                subtree: false
            };

            // Use mutation observers to figure out if the row is selected or not
            // 
            function subscriber(mutations) {
                mutations.forEach((mutation) => {
                    // handle mutations here
                    this.set("isSelected", mutation.target.classList.contains("dgrid-selected") && mutation.target.classList.contains("dgrid-focus"));
                });
            }

            // instantiating observer
            this._observer = new MutationObserver(subscriber.bind(this));

            // observing target
            this._observer.observe(this.domNode, config);
        },

        destroy: function() {
            // Disconnect the observer
            if(this._observer) {
                this._observer.disconnect();
                this._observer = null;
            }

            if(this._source) {
                this._source.destroy();
            }
            if(this._cropsDndSource) {
                this._cropsDndSource.destroy();
            }

            this.inherited(arguments);
        },

        _onInfoClick: function(){
            when(this.contentDataStore.get(this.item.contentLink))
                .then(function(item) {
                    this._showTooltip(item);
                }.bind(this));
        },

        _showTooltip: function (item) {
            if (this._isPopupOpened) {
                this._closePopup();
                return;
            }

            var typespecificcontent = lang.replace('<div class="headline">{0}</div><p>{1}px</p><div class="headline">{2}</div><p>{3}px</p>', [
                this.labelInfo.width,
                item.properties.imageWidth,
                this.labelInfo.height,
                item.properties.imageHeight]);
            if (item.properties.assetType !== 4) {
                typespecificcontent = '';
            } else {
                if (item.properties.cropName !== null) {
                    typespecificcontent = '<div class="headline">' + this.labelInfo.cropName + '</div><p>' + item.properties.cropName + '</p>' + typespecificcontent;
                }
            }
            var sizestr = item.properties.assetSize + " b";
            if (item.properties.assetSize >= 1000 && item.properties.assetSize < 1000000) {
                sizestr = Math.round(item.properties.assetSize / 1000) + " kb";
            } else if (item.properties.assetSize >= 1000000) sizestr = Math.round(item.properties.assetSize / 1000000) + " mb";
            var mmurl = item.properties.mmUrl;
            var content = lang.replace(ContentItemTooltipTemplate, [
                this.labelInfo.title,
                item.name,
                item.properties.description === null ? '' : item.properties.description,
                this.item.thumbnailUrl,
                this.labelInfo.assetType,
                item.properties.assetTypeName,
                this.labelInfo.keywords,
                item.properties.keywords === null ? '' : item.properties.keywords.join(', '),
                this.labelInfo.size,
                sizestr,
                typespecificcontent,
                mmurl,
                mmurl === '' ? 'display:none' : '',
                this.labelInfo.mediaManagerUrl
            ]);
            this.dialog = this.dialog || new TooltipDialog({
                baseClass: "digi-dijitTooltipDialog epi-dijitTooltipDialog dijitTooltipDialog",
                connectorMargin: 0,
                content: content
            });
            this.own(this.dialog);
            
            !this.dialog._started  && this.dialog.startup();

            popup.open({
                parent: this,
                popup: this.dialog,
                around: this.rowNode,
                orient: ["before", "above", "above-alt" , "below", "below-alt"]
            });
            this._isPopupOpened = true;
        },

        _onBlur: function () {
            this._isPopupOpened && this._closePopup();

            this.inherited(arguments);
        },

        _closePopup: function() {
            popup.close(this.dialog);
            this._isPopupOpened = false;
        },
        _setIsSelectedAttr: function (isSelected) {
            this._set("isSelected", isSelected);

            if(!this.isSelected || !this.item.thumbnailUrl){
                this._toggleCrops(false);
                return;
            }

            this._loadCrops()
                .then(function (crops) {
                    if (!crops.length) {
                        this._toggleCrops(false);
                        return;
                    }
                    this._renderCrops(crops);
                    this._toggleCrops(true);
                }.bind(this));
        },
        _toggleCrops: function (visible) {
            domClass.toggle(this.cropsNode, "dijitHidden", !visible);
            domClass.toggle(this.arrowNode, "dijitHidden", !visible);
        },
        _loadCrops: function () {
            if(this.siblings) {
                return when(this.siblings);
            }
            return when(this.store.query({ query: "DigiGetSiblings", references: [this.item.contentLink] }))
                .then(function (siblings) {
                    this.siblings = siblings || [];
                    return this.siblings;
                }.bind(this));
        },
        _renderCrops: function (crops) {
            // Only render the crop nodes once
            if(crops.length === 0 || this._cropsRendered) {
                return;
            }

            crops.forEach(function(crop){
                var cropNode = domConstruct.toDom('<li id="crop_' + crop.contentLink + '" style="float:left;" display="block" class="dojoDndItem digi-content-list__content-item__crops__crop" title="' + crop.name + '"><img class="digi-content-list__content-item__crops__crop__image" src="' + crop.thumbnailUrl + '" display="block"/></li>');
                
                // Add each crop to the dnd source
                this._cropsDndSource.setItem(cropNode.id, {
                    name: crop.name,
                    data: crop,
                    type: this._dnd._getDndType(crop)
                });
                domConstruct.place(cropNode, this.cropsNode);
            }.bind(this));
            this._cropsRendered = true;

            // Sync the nodes in the crop dnd source
            this._cropsDndSource.sync();
        },
        _createDndItem: function (item) {
            var node = domConstruct.create("div");
            var types = this._dnd._getDndType(item);
            return {
                node: node,
                type: types,
                data: item
            };
        }
    });

    return declare([EpiContentList], {

        _gridClass: declare([OnDemandList, Selection,  SelectionExtensions, Keyboard, WithContextMenu, DijitRegistry, WidgetRow]),

        _translationStore: null,
        _labelsInfo: null,

        postMixInProperties: function () {
            this.inherited(arguments);

            var registry = dependency.resolve("epi.storeregistry");
            this._store = this._store || registry.get("epi.cms.contentdata");
            this._translationStore = this._translationStore || registry.get("digizuite.translation");
        },

        postCreate: function () {
            this.inherited(arguments);

            when(this._translationStore.executeMethod('GetTranslationsAssetInfo', ''),
                function (response) {
                    this._labelsInfo = response;
                }.bind(this)); 

            // Hook up the render row on the grid
            this.grid.renderRow = this._renderRow.bind(this);

            // Set the min and max rows per page to the same value
            // so it is possible to mimic regular paging in the query
            this.grid.maxRowsPerPage = 25;
            this.grid.minRowsPerPage = 25;
        },

        _renderRow: function (contentItem) {
            var item = new ContentItem({ item: contentItem, store: this.store, contentDataStore: this._store, labelInfo: this._labelsInfo });
            return item.domNode;
        }
    });
});