var tinymce = tinymce || {};
tinymce.PluginManager.add("digi-image-tools", function (editor) {

    var mediaFormats = editor.settings["digi-media-formats"];

    var imageNode;
    editor.addCommand("mceDigiMediaTypeSelector", function () {
        require(["epi/shell/DialogService", "digizuite/tinymce/Selector"], function (DialogService, Selector) {
            editor.fire("OpenWindow", {
                win: null
            });

            var selectedNode = imageNode;
            var imgSrc = editor.dom.getAttrib(selectedNode, "src");

            var selector = new Selector({
                mediaFormats: mediaFormats,
                image: imgSrc
            });
            selector.startup();
            DialogService.dialog({ title: "Select media format", dialogClass: "epi-dialog-confirm digi-mediaformat-dialog", defaultActionsVisible: false, content: selector })
                .then(function () {
                    editor.dom.setAttribs(imageNode, { "src": selector.get("image"), "width": "", "height": "" });
                    var img = new Image();
                    img.onload = function () {
                        var w = img.width;
                        var h = img.height;
                        editor.dom.setAttribs(imageNode, { "width": w, "height": h });
                    };
                    img.src = selector.get("image");


                    //Wrap all changes as one undo level
                    editor.undoManager.add();
                })
                .always(function () {
                    selector && selector.destroy();

                    editor.fire("CloseWindow", {
                        win: null
                    });
                });
        });
    });

    editor.addButton("digi-image-tools", {
        title: "Digizuite media type selector",
        cmd: "mceDigiMediaTypeSelector",
        image: editor.settings["digi-media-formats-icon"],
        onPostRender: function () {
            // Add a node change handler, selects the button in the UI when a image is selected
            editor.on("NodeChange", function (e) {
                //Prevent tool from being activated as objects represented as images.
                var isStandardImage = e.element.tagName === "IMG" && editor.dom.getAttrib(e.element, "class").indexOf("mceItem") === -1;
                var editImageDiv=document.querySelector(".mce-i-editimage").closest("div");
                //Set or reset imageNode to the node cause Image Editor command enabled
                imageNode = isStandardImage ? e.element : null;
                this.active(isStandardImage);
                this.disabled(true);
                editImageDiv.style.display = '';
                if (!isStandardImage) { return; }

                // Enable the button if it is a digi image
                require(["dojo/when", "epi/Url", "epi-cms/core/PermanentLinkHelper"], function (when, Url, PermanentLinkHelper) {
                    var url = new Url(imageNode.src);
                    when(PermanentLinkHelper.getContent(url.path)).then(function (content) {
                        if (!content || !content.capabilities.isDigizuiteImage) {
                            this.disabled(true);
                            editImageDiv.style.display = '';
                        } else {
                            this.disabled(false);
                            editImageDiv.style.display = 'none';
                        }
                    }.bind(this));
                }.bind(this));
            }.bind(this));
        }
    });

    return {
        getMetadata: function () {
            return {
                name: "Digizuite media type selector",
                url: "https://www.digizuite.com"
            };
        }
    };
});
